import streamlit
import sys
from streamlit.web import cli as stcli

if __name__ == '__main__':
    streamlit._is_running_with_streamlit = True
    sys.argv = ["streamlit", "run", "main.py", "--server.port=8000"]
    sys.exit(stcli.main())